# Google
GOOGLE_TOKEN_URL = 'https://accounts.google.com/o/oauth2/token'
GOOGLE_AUTH_URL = 'https://accounts.google.com/o/oauth2/auth'
GOOGLE_USER_URL = 'https://www.googleapis.com/oauth2/v2/userinfo'
# Facebook
FACEBOOK_TOKEN_URL = 'https://graph.facebook.com/oauth/access_token'
FACEBOOK_AUTH_URL = 'https://www.facebook.com/v2.12/dialog/oauth?'
FACEBOOK_USER_URL = 'https://graph.facebook.com/v2.11/me'
